@@ -41,6 +41,7 @@ class FilterReg():
         target_normals (numpy.ndarray, optional): Normals of target points.
         sigma2 (Float, optional): Variance parameter. If this variable is None,
             the variance is updated in Mstep.
+        update_sigma2 (bool, optional): If this variable is True, Update sigma2 in the registration iteration.
     """
     def __init__(self, source=None, target_normals=None,
                  sigma2=None, update_sigma2=False):
@@ -145,7 +146,7 @@ class RigidFilterReg(FilterReg):
 
     @staticmethod
     def _maximization_step(t_source, target, estep_res, trans_p, sigma2, w=0.0,
-                           objective_type='pt2pt', maxiter=10, tol=1.0e-4):
+                           objective_type='pt2pt'):
         m, dim = t_source.shape
         n = target.shape[0]
         assert dim == 2 or dim == 3, "dim must be 2 or 3."
@@ -161,9 +162,9 @@ class RigidFilterReg(FilterReg):
                 dr, dt = kabsch.kabsch2d(t_source, m1m0, drxdx)
             else:
                 dr, dt = kabsch.kabsch(t_source, m1m0, drxdx)
-            rx = np.multiply(drxdx, (t_source - m1m0).T).T.sum(axis=1)
+            rx = np.multiply(drxdx, (t_source - m1m0).T).T
             rot, t = np.dot(dr, trans_p.rot), np.dot(trans_p.t, dr.T) + dt
-            q = np.dot(rx.T, rx).sum()
+            q = np.linalg.norm(rx, ord=2, axis=1).sum()
         elif objective_type == 'pt2pl':
             nxm0 = (nx.T / m0).T
             tw, q = pt2pl.compute_twist_for_pt2pl(t_source, m1m0, nxm0, drxdx)
@@ -266,7 +267,7 @@ def registration_filterreg(source, target, target_normals=None,
         callback (:obj:`list` of :obj:`function`, optional): Called after each iteration.
             `callback(probreg.Transformation)`
 
-    Kwargs:
+    Keyword Args:
         tf_init_params (dict, optional): Parameters to initialize transformation (for rigid).
     """
     cv = lambda x: np.asarray(x.points if isinstance(x, o3.geometry.PointCloud) else x)
